@@ -5,7 +5,7 @@ use warnings;
 package MooX::Press;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.031';
+our $VERSION   = '0.032';
 
 use Types::Standard 1.008003 -is, -types;
 use Types::TypeTiny qw(ArrayLike HashLike);
@@ -1053,25 +1053,38 @@ sub require_methods_mouse {
 	(Mouse::Util::find_meta($role) or $role->meta)->add_required_methods(sort keys %$methods);
 }
 
+sub wrap_coderef {
+	my $builder = shift;
+	my %method  = (@_==1) ? %{$_[0]} : @_;
+	my $qname   = delete($method{package}) || caller;
+	$method{lexical} = !!1;
+	my $return = $builder->install_methods($qname, { '__ANON__' => \%method });
+	$return->{'__ANON__'};
+}
+
 sub install_methods {
 	my $builder = shift;
 	my ($class, $methods) = @_;
+	my %return;
 	for my $name (sort keys %$methods) {
 		no strict 'refs';
-		my ($code, $signature, $signature_style, $invocant_count, @curry);
+		my ($code, $signature, $signature_style, $invocant_count, $is_coderef, $caller, @curry);
+		$caller = $class;
 		
 		if (is_CodeRef($methods->{$name})) {
 			$code            = $methods->{$name};
 			$signature_style = 'none';
 		}
 		elsif (is_HashRef($methods->{$name})) {
-			$code      = $methods->{$name}{code};
-			$signature = $methods->{$name}{signature};
-			@curry     = @{ $methods->{$name}{curry} || [] };
+			$code       = $methods->{$name}{code};
+			$signature  = $methods->{$name}{signature};
+			@curry      = @{ $methods->{$name}{curry} || [] };
 			$invocant_count  = exists($methods->{$name}{invocant_count}) ? $methods->{$name}{invocant_count} : 1;
 			$signature_style = is_CodeRef($signature)
 				? 'code'
 				: ($methods->{$name}{named} ? 'named' : 'positional');
+			$is_coderef = !!$methods->{$name}{lexical};
+			$caller     = $methods->{$name}{caller};
 		}
 		
 		if ($signature) {
@@ -1089,6 +1102,15 @@ sub install_methods {
 			}
 		}
 		
+		my $callcode;
+		if (is_CodeRef($code)) {
+			$callcode = 'goto $code';
+		}
+		else {
+			($callcode = $code) =~ s/sub/do/;
+			$callcode = "package $caller; $callcode" if defined $caller;
+		}
+		
 		my $subcode = sprintf(
 			q{
 				package %-49s  # package name
@@ -1100,13 +1122,13 @@ sub install_methods {
 					%-49s          # reassemble @_ from @invocants, @curry, and &$check
 					%-49s          # run sub code
 				};
-				1;
+				%s
 			},
 			"$class;",
 			(($signature && !$optimized)
 				? 'my $check;'
 				: ''),
-			"$name :method",
+			($is_coderef ? '' : "$name :method"),
 			($signature
 				? sprintf('my @invocants = splice(@_, 0, %d);', $invocant_count)
 				: ''),
@@ -1116,11 +1138,13 @@ sub install_methods {
 			($signature
 				? (@curry ? sprintf('@_ = (@invocants, @curry, %s);', $checkcode) : sprintf('@_ = (@invocants, %s);', $checkcode))
 				: (@curry ? sprintf('splice(@_, %d, 0, @curry);', $invocant_count) : '')),
-			(is_CodeRef($code) ? 'goto $code' : do { (my $callcode = $code) =~ s/sub/do/; $callcode }),
+			$callcode,
+			($is_coderef ? '' : '1;'),
 		);
-		eval($subcode)
+		($return{$name} = eval($subcode))
 			or $builder->croak("Could not create method $name in package $class: $@");
 	}
+	\%return;
 }
 
 sub _optimize_signature {
